@@ -1,6 +1,6 @@
 /*
  * iostat: report CPU and I/O statistics
- * (C) 1999-2019 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2023 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _IOSTAT_H
@@ -15,13 +15,13 @@
 #define I_D_EXTENDED		0x000008
 #define I_D_EVERYTHING		0x000010
 #define I_D_KILOBYTES		0x000020
-/* Unused			0x000040 */
+#define I_D_ALL_DIR		0x000040
 #define I_D_DEBUG		0x000080
 #define I_D_UNFILTERED		0x000100
 #define I_D_MEGABYTES		0x000200
 #define I_D_ALL_DEVICES		0x000400
 #define I_F_GROUP_DEFINED	0x000800
-#define I_D_HUMAN_READ		0x001000
+#define I_D_PRETTY		0x001000
 #define I_D_PERSIST_NAME	0x002000
 #define I_D_OMIT_SINCE_BOOT	0x004000
 #define I_D_JSON_OUTPUT		0x008000
@@ -31,6 +31,7 @@
 #define I_D_ZERO_OMIT		0x080000
 #define I_D_UNIT		0x100000
 #define I_D_SHORT_OUTPUT	0x200000
+#define I_D_COMPACT		0x400000
 
 #define DISPLAY_CPU(m)			(((m) & I_D_CPU)              == I_D_CPU)
 #define DISPLAY_DISK(m)			(((m) & I_D_DISK)             == I_D_DISK)
@@ -43,7 +44,7 @@
 #define DISPLAY_UNFILTERED(m)		(((m) & I_D_UNFILTERED)       == I_D_UNFILTERED)
 #define DISPLAY_ALL_DEVICES(m)		(((m) & I_D_ALL_DEVICES)      == I_D_ALL_DEVICES)
 #define GROUP_DEFINED(m)		(((m) & I_F_GROUP_DEFINED)    == I_F_GROUP_DEFINED)
-#define DISPLAY_HUMAN_READ(m)		(((m) & I_D_HUMAN_READ)       == I_D_HUMAN_READ)
+#define DISPLAY_PRETTY(m)		(((m) & I_D_PRETTY)           == I_D_PRETTY)
 #define DISPLAY_PERSIST_NAME_I(m)	(((m) & I_D_PERSIST_NAME)     == I_D_PERSIST_NAME)
 #define DISPLAY_OMIT_SINCE_BOOT(m)	(((m) & I_D_OMIT_SINCE_BOOT)  == I_D_OMIT_SINCE_BOOT)
 #define DISPLAY_DEVMAP_NAME(m)		(((m) & I_D_DEVMAP_NAME)      == I_D_DEVMAP_NAME)
@@ -53,12 +54,16 @@
 #define DISPLAY_JSON_OUTPUT(m)		(((m) & I_D_JSON_OUTPUT)      == I_D_JSON_OUTPUT)
 #define DISPLAY_UNIT(m)			(((m) & I_D_UNIT)	      == I_D_UNIT)
 #define DISPLAY_SHORT_OUTPUT(m)		(((m) & I_D_SHORT_OUTPUT)     == I_D_SHORT_OUTPUT)
+#define USE_ALL_DIR(m)			(((m) & I_D_ALL_DIR)          == I_D_ALL_DIR)
+#define DISPLAY_COMPACT(m)		(((m) & I_D_COMPACT)          == I_D_COMPACT)
 
 #define T_PART		0
 #define T_DEV		1
 #define T_PART_DEV	2
 #define T_GROUP		3
 
+#define UKWN_MAJ_NR	0
+
 /* Environment variable */
 #define ENV_POSIXLY_CORRECT	"POSIXLY_CORRECT"
 
@@ -85,12 +90,16 @@ struct io_stats {
 	unsigned long dc_ios		__attribute__ ((packed));
 	/* # of discard requests merged */
 	unsigned long dc_merges		__attribute__ ((packed));
+	/* # of flush requests issued to the device */
+	unsigned long fl_ios		__attribute__ ((packed));
 	/* Time of read requests in queue */
 	unsigned int  rd_ticks		__attribute__ ((packed));
 	/* Time of write requests in queue */
 	unsigned int  wr_ticks		__attribute__ ((packed));
 	/* Time of discard requests in queue */
 	unsigned int  dc_ticks		__attribute__ ((packed));
+	/* Time of flush requests in queue */
+	unsigned int  fl_ticks		__attribute__ ((packed));
 	/* # of I/Os in progress */
 	unsigned int  ios_pgr		__attribute__ ((packed));
 	/* # of ticks total (for this device) for I/O */
@@ -112,7 +121,7 @@ struct io_device {
 	int dev_tp;
 	/* TRUE if device exists in /proc/diskstats or /sys. Don't apply for groups. */
 	int exist;
-	/* major and minor numbers are set only for partitions (T_PART), not whole devices */
+	/* major and minor numbers (not set for T_GROUP "devices") */
 	int major;
 	int minor;
 	struct io_stats *dev_stats[2];
@@ -126,6 +135,8 @@ struct ext_io_stats {
 	double w_await;
 	/* d_await */
 	double d_await;
+	/* f_await */
+	double f_await;
 	/* rsec/s */
 	double rsectors;
 	/* wsec/s */
