@@ -1,6 +1,6 @@
 /*
- * pcp_stats.c: Funtions used by sadf to create PCP archive files.
- * (C) 2019-2021 by Sebastien GODARD (sysstat <at> orange.fr)
+ * pcp_stats.c: Functions used by sadf to create PCP archive files.
+ * (C) 2019-2023 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -139,13 +139,13 @@ __print_funct_t pcp_print_cpu_stats(struct activity *a, int curr)
 		pmiPutValue(i ? "kernel.percpu.cpu.sys" : "kernel.all.cpu.sys", str, buf);
 
 		snprintf(buf, sizeof(buf), "%llu", scc->cpu_iowait);
-		pmiPutValue(i ? "kernel.percpu.cpu.iowait" : "kernel.all.cpu.wait.total", str, buf);
+		pmiPutValue(i ? "kernel.percpu.cpu.iowait" : "kernel.all.cpu.iowait", str, buf);
 
 		snprintf(buf, sizeof(buf), "%llu", scc->cpu_steal);
 		pmiPutValue(i ? "kernel.percpu.cpu.steal" : "kernel.all.cpu.steal", str, buf);
 
 		snprintf(buf, sizeof(buf), "%llu", scc->cpu_hardirq + scc->cpu_softirq);
-		pmiPutValue(i ? "kernel.percpu.cpu.intr" : "kernel.all.cpu.intr", str, buf);
+		pmiPutValue(i ? "kernel.percpu.cpu.irq.total" : "kernel.all.cpu.irq.total", str, buf);
 
 		snprintf(buf, sizeof(buf), "%llu", scc->cpu_hardirq);
 		pmiPutValue(i ? "kernel.percpu.cpu.irq.hard" : "kernel.all.cpu.irq.hard", str, buf);
@@ -201,26 +201,60 @@ __print_funct_t pcp_print_pcsw_stats(struct activity *a, int curr)
 __print_funct_t pcp_print_irq_stats(struct activity *a, int curr)
 {
 #ifdef HAVE_PCP
-	char buf[64], intno[64];
-	int i;
-	struct stats_irq *sic;
+	int i, c;
+	char buf[64], name[64];
+	struct stats_irq *stc_cpu_irq, *stc_cpuall_irq;
+	unsigned char masked_cpu_bitmap[BITMAP_SIZE(NR_CPUS)] = {0};
 
-	for (i = 0; (i < a->nr[curr]) && (i < a->bitmap->b_size + 1); i++) {
+	/* @nr[curr] cannot normally be greater than @nr_ini */
+	if (a->nr[curr] > a->nr_ini) {
+		a->nr_ini = a->nr[curr];
+	}
 
-		sic = (struct stats_irq *) ((char *) a->buf[curr]  + i * a->msize);
+	/* Identify offline and unselected CPU, and keep persistent statistics values */
+	get_global_int_statistics(a, !curr, curr, flags, masked_cpu_bitmap);
 
-		/* Should current interrupt (including int "sum") be displayed? */
-		if (a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) {
+	for (i = 0; i < a->nr2; i++) {
 
-			if (!i) {
-				/* This is interrupt "sum" */
-				snprintf(buf, sizeof(buf), "%llu", sic->irq_nr);
-				pmiPutValue("kernel.all.intr", NULL, buf);
+		stc_cpuall_irq = (struct stats_irq *) ((char *) a->buf[curr] + i * a->msize);
+
+		if (a->item_list != NULL) {
+			/* A list of interrupts has been entered on the command line */
+			if (!search_list_item(a->item_list, stc_cpuall_irq->irq_name))
+				/* Interrupt not found in list */
+				continue;
+		}
+
+		for (c = 0; (c < a->nr[curr]) && (c < a->bitmap->b_size + 1); c++) {
+
+			stc_cpu_irq = (struct stats_irq *) ((char *) a->buf[curr] + c * a->msize * a->nr2
+										  + i * a->msize);
+
+			/* Should current CPU (including CPU "all") be processed? */
+			if (masked_cpu_bitmap[c >> 3] & (1 << (c & 0x07)))
+				/* No */
+				continue;
+
+			snprintf(buf, sizeof(buf), "%u", stc_cpu_irq->irq_nr);
+
+			if (!c) {
+				/* This is CPU "all" */
+				if (!i) {
+					/* This is interrupt "sum" */
+					pmiPutValue("kernel.all.intr", NULL, buf);
+				}
+				else {
+					pmiPutValue("kernel.all.interrupts.total",
+						    stc_cpuall_irq->irq_name, buf);
+				}
 			}
 			else {
-				snprintf(intno, sizeof(intno), "int%d", i - 1);
-				snprintf(buf, sizeof(buf), "%llu", sic->irq_nr);
-				pmiPutValue("kernel.all.interrupts.total", intno, buf);
+				/* This is a particular CPU */
+				snprintf(name, sizeof(name), "%s::cpu%d",
+					 stc_cpuall_irq->irq_name, c - 1);
+				name[sizeof(name) - 1] = '\0';
+
+				pmiPutValue("kernel.percpu.interrupts", name, buf);
 			}
 		}
 	}
@@ -368,7 +402,7 @@ __print_funct_t pcp_print_memory_stats(struct activity *a, int curr)
 		pmiPutValue("mem.util.used", NULL, buf);
 
 		snprintf(buf, sizeof(buf), "%llu", smc->bufkb);
-		pmiPutValue("mem.util.buffers", NULL, buf);
+		pmiPutValue("mem.util.bufmem", NULL, buf);
 
 		snprintf(buf, sizeof(buf), "%llu", smc->camkb);
 		pmiPutValue("mem.util.cached", NULL, buf);
@@ -394,13 +428,13 @@ __print_funct_t pcp_print_memory_stats(struct activity *a, int curr)
 			pmiPutValue("mem.util.slab", NULL, buf);
 
 			snprintf(buf, sizeof(buf), "%llu", smc->kstackkb);
-			pmiPutValue("mem.util.stack", NULL, buf);
+			pmiPutValue("mem.util.kernelStack", NULL, buf);
 
 			snprintf(buf, sizeof(buf), "%llu", smc->pgtblkb);
 			pmiPutValue("mem.util.pageTables", NULL, buf);
 
 			snprintf(buf, sizeof(buf), "%llu", smc->vmusedkb);
-			pmiPutValue("mem.util.vmused", NULL, buf);
+			pmiPutValue("mem.util.vmallocUsed", NULL, buf);
 		}
 	}
 
@@ -1087,10 +1121,10 @@ __print_funct_t pcp_print_net_etcp_stats(struct activity *a, int curr)
 	pmiPutValue("network.tcp.attemptfails", NULL, buf);
 
 	snprintf(buf, sizeof(buf), "%llu", (unsigned long long) snetc->EstabResets);
-	pmiPutValue("network.tcp.tcpestabresets", NULL, buf);
+	pmiPutValue("network.tcp.estabresets", NULL, buf);
 
 	snprintf(buf, sizeof(buf), "%llu", (unsigned long long) snetc->RetransSegs);
-	pmiPutValue("network.tcp.tcpretranssegs", NULL, buf);
+	pmiPutValue("network.tcp.retranssegs", NULL, buf);
 
 	snprintf(buf, sizeof(buf), "%llu", (unsigned long long) snetc->InErrs);
 	pmiPutValue("network.tcp.inerrs", NULL, buf);
@@ -1754,6 +1788,9 @@ __print_funct_t pcp_print_softnet_stats(struct activity *a, int curr)
 
 		snprintf(buf, sizeof(buf), "%llu", (unsigned long long) ssnc->flow_limit);
 		pmiPutValue("network.softnet.percpu.flow_limit", cpuno, buf);
+
+		snprintf(buf, sizeof(buf), "%llu", (unsigned long long) ssnc->backlog_len);
+		pmiPutValue("network.softnet.percpu.backlog_length", cpuno, buf);
 	}
 #endif	/* HAVE_PCP */
 }
