@@ -1,6 +1,6 @@
 /*
  * rd_stats.h: Include file used to read system statistics
- * (C) 1999-2019 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2023 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _RD_STATS_H
@@ -14,16 +14,10 @@
  ***************************************************************************
  */
 
-/* Get IFNAMSIZ */
-#include <net/if.h>
-#ifndef IFNAMSIZ
-#define IFNAMSIZ	16
-#endif
-
 /* Maximum length of block device name */
 #define MAX_DEV_LEN	128
 /* Maximum length of network interface name */
-#define MAX_IFACE_LEN	IFNAMSIZ
+#define MAX_IFACE_LEN	16
 /* Maximum length of USB manufacturer string */
 #define MAX_MANUF_LEN	24
 /* Maximum length of USB product string */
@@ -32,6 +26,8 @@
 #define MAX_FS_LEN	128
 /* Maximum length of FC host name */
 #define MAX_FCH_LEN	16
+/* Maximum length of interrupt name */
+#define MAX_SA_IRQ_LEN	8
 
 #define CNT_PART	1
 #define CNT_ALL_DEV	0
@@ -65,6 +61,10 @@
 #define NET_RPC_NFSD	PRE "/proc/net/rpc/nfsd"
 #define NET_SOFTNET	PRE "/proc/net/softnet_stat"
 #define LOADAVG		PRE "/proc/loadavg"
+#define PRESSURE	PRE "/proc/pressure"
+#define PSI_CPU		PRESSURE "/cpu"
+#define PSI_IO		PRESSURE "/io"
+#define PSI_MEM		PRESSURE "/memory"
 #define VMSTAT		PRE "/proc/vmstat"
 #define NET_SNMP	PRE "/proc/net/snmp"
 #define NET_SNMP6	PRE "/proc/net/snmp6"
@@ -95,8 +95,10 @@
 
 /*
  * Structure for CPU statistics.
- * In activity buffer: First structure is for global CPU utilisation ("all").
+ * In activity buffer: First structure is for global CPU utilization ("all").
  * Following structures are for each individual CPU (0, 1, etc.)
+ *
+ * Used by: sadc, sar, sadf, iostat, mpstat, pidstat
  */
 struct stats_cpu {
 	unsigned long long cpu_user;
@@ -118,7 +120,7 @@ struct stats_cpu {
 
 /*
  * Structure for task creation and context switch statistics.
- * The attribute (aligned(16)) is necessary so that sizeof(structure) has
+ * The attribute (aligned(8)) is necessary so that sizeof(structure) has
  * the same value on 32 and 64-bit architectures.
  */
 struct stats_pcsw {
@@ -133,17 +135,21 @@ struct stats_pcsw {
 
 /*
  * Structure for interrupts statistics.
- * In activity buffer: First structure is for total number of interrupts ("SUM").
- * Following structures are for each individual interrupt (0, 1, etc.)
+ * In activity buffer (sadc): First structures are for global CPU utilization ("all"):
+ * interrupts sum, 0, 1,...
+ * Following structures are for each individual CPU (0, 1, etc.)
+ *
+ * Used by: sadc, sar, sadf
  */
 struct stats_irq {
-	unsigned long long irq_nr;
+	unsigned int irq_nr;
+	char	     irq_name[MAX_SA_IRQ_LEN];
 };
 
 #define STATS_IRQ_SIZE	(sizeof(struct stats_irq))
-#define STATS_IRQ_ULL	1
+#define STATS_IRQ_ULL	0
 #define STATS_IRQ_UL	0
-#define STATS_IRQ_U	0
+#define STATS_IRQ_U	1
 
 /* Structure for swapping statistics */
 struct stats_swap {
@@ -189,7 +195,11 @@ struct stats_io {
 #define STATS_IO_UL	0
 #define STATS_IO_U	0
 
-/* Structure for memory and swap space utilization statistics */
+/*
+ * Structure for memory and swap space utilization statistics.
+ *
+ * Used by: sadc, sar, sadf, pidstat
+ */
 struct stats_memory {
 	unsigned long long frmkb;
 	unsigned long long bufkb;
@@ -259,7 +269,11 @@ struct stats_serial {
 #define STATS_SERIAL_UL		0
 #define STATS_SERIAL_U		7
 
-/* Structure for block devices statistics */
+/*
+ * Structure for block devices statistics.
+ *
+ * Used by: sadc, sar, sadf, iostat
+ */
 struct stats_disk {
 	unsigned long long nr_ios;
 	unsigned long long wwn[2];
@@ -602,7 +616,7 @@ struct stats_net_udp6 {
 
 /*
  * Structure for CPU frequency statistics.
- * In activity buffer: First structure is for global CPU utilisation ("all").
+ * In activity buffer: First structure is for global CPU utilization ("all").
  * Following structures are for each individual CPU (0, 1, etc.)
  */
 struct stats_pwr_cpufreq {
@@ -629,7 +643,7 @@ struct stats_huge {
 
 /*
  * Structure for weighted CPU frequency statistics.
- * In activity buffer: First structure is for global CPU utilisation ("all").
+ * In activity buffer: First structure is for global CPU utilization ("all").
  * Following structures are for each individual CPU (0, 1, etc.)
  */
 struct stats_pwr_wghfreq {
@@ -697,12 +711,60 @@ struct stats_softnet {
 	unsigned int time_squeeze;
 	unsigned int received_rps;
 	unsigned int flow_limit;
+	unsigned int backlog_len;
 };
 
 #define STATS_SOFTNET_SIZE	(sizeof(struct stats_softnet))
 #define STATS_SOFTNET_ULL	0
 #define STATS_SOFTNET_UL	0
-#define STATS_SOFTNET_U		5
+#define STATS_SOFTNET_U		6
+
+/* Structure for pressure-stall CPU statistics */
+struct stats_psi_cpu {
+	unsigned long long some_cpu_total;
+	unsigned long	   some_acpu_10		__attribute__ ((aligned (8)));
+	unsigned long	   some_acpu_60		__attribute__ ((aligned (8)));
+	unsigned long	   some_acpu_300	__attribute__ ((aligned (8)));
+};
+
+#define STATS_PSI_CPU_SIZE	(sizeof(struct stats_psi_cpu))
+#define STATS_PSI_CPU_ULL	1
+#define STATS_PSI_CPU_UL	3
+#define STATS_PSI_CPU_U		0
+
+/* Structure for pressure-stall I/O statistics */
+struct stats_psi_io {
+	unsigned long long some_io_total;
+	unsigned long long full_io_total;
+	unsigned long	   some_aio_10		__attribute__ ((aligned (8)));
+	unsigned long	   some_aio_60		__attribute__ ((aligned (8)));
+	unsigned long	   some_aio_300		__attribute__ ((aligned (8)));
+	unsigned long	   full_aio_10		__attribute__ ((aligned (8)));
+	unsigned long	   full_aio_60		__attribute__ ((aligned (8)));
+	unsigned long	   full_aio_300		__attribute__ ((aligned (8)));
+};
+
+#define STATS_PSI_IO_SIZE	(sizeof(struct stats_psi_io))
+#define STATS_PSI_IO_ULL	2
+#define STATS_PSI_IO_UL		6
+#define STATS_PSI_IO_U		0
+
+/* Structure for pressure-stall memory statistics */
+struct stats_psi_mem {
+	unsigned long long some_mem_total;
+	unsigned long long full_mem_total;
+	unsigned long	   some_amem_10		__attribute__ ((aligned (8)));
+	unsigned long	   some_amem_60		__attribute__ ((aligned (8)));
+	unsigned long	   some_amem_300	__attribute__ ((aligned (8)));
+	unsigned long	   full_amem_10		__attribute__ ((aligned (8)));
+	unsigned long	   full_amem_60		__attribute__ ((aligned (8)));
+	unsigned long	   full_amem_300	__attribute__ ((aligned (8)));
+};
+
+#define STATS_PSI_MEM_SIZE	(sizeof(struct stats_psi_mem))
+#define STATS_PSI_MEM_ULL	2
+#define STATS_PSI_MEM_UL	6
+#define STATS_PSI_MEM_U		0
 
 /*
  ***************************************************************************
@@ -718,7 +780,7 @@ unsigned long long get_per_cpu_interval
 __nr_t read_stat_cpu
 	(struct stats_cpu *, __nr_t);
 __nr_t read_stat_irq
-	(struct stats_irq *, __nr_t);
+	(struct stats_irq *, __nr_t, __nr_t);
 __nr_t read_meminfo
 	(struct stats_memory *);
 void read_uptime
@@ -794,6 +856,13 @@ __nr_t read_fchost
 	(struct stats_fchost *, __nr_t);
 int read_softnet
 	(struct stats_softnet *, __nr_t, unsigned char []);
+__nr_t read_psicpu
+	(struct stats_psi_cpu *);
+__nr_t read_psiio
+	(struct stats_psi_io *);
+__nr_t read_psimem
+	(struct stats_psi_mem *);
+
 #endif /* SOURCE_SADC */
 
 #endif /* _RD_STATS_H */
