@@ -1,6 +1,6 @@
 /*
  * sar/sadc: Report system activity
- * (C) 1999-2021 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2023 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _SA_H
@@ -25,7 +25,7 @@
 #define MAX_NR_ACT	256
 
 /* Number of functions used to count items */
-#define NR_F_COUNT	12
+#define NR_F_COUNT	13
 
 /* Activities */
 #define A_CPU		1
@@ -288,7 +288,7 @@
 
 /* NR_MAX is the upper limit used for unknown activities */
 #define NR_MAX		(65536 * 4096)
-#define NR2_MAX		1024
+#define NR2_MAX		4096
 
 /* Maximum number of args that can be passed to sadc */
 #define MAX_ARGV_NR	32
@@ -299,6 +299,7 @@
 #define NO_TM_START		0
 #define NO_TM_END		0
 #define NO_RESET		0
+#define NO_RANGE		0
 #define NON_FATAL		0
 #define FATAL			1
 #define C_SAR			0
@@ -634,11 +635,6 @@ struct file_header {
  * Base magical number for activities.
  */
 #define ACTIVITY_MAGIC_BASE	0x8a
-/*
- * Magical value used for activities with
- * unknown format (used for sadf -H only).
- */
-#define ACTIVITY_MAGIC_UNKNOWN	0x89
 
 /* List of activities saved in file */
 struct file_activity {
@@ -931,15 +927,15 @@ struct activity {
 	 */
 	int f_count_index;
 	/*
-	 * The f_count2() function is used to count the number of
-	 * sub-items -> @nr2
+	 * Index in f_count[] array to determine function used to count the
+	 * number of sub-items -> @nr2
 	 * Such a function should _always_ return a value greater than
 	 * or equal to 0.
 	 *
-	 * A NULL value for this function pointer indicates that the number of items
-	 * is a constant (and @nr2 is set to this value).
+	 * A value of -1 indicates that the number of items is a constant
+	 * (and @nr2 is set to this value).
 	 */
-	__nr_t (*f_count2) (struct activity *);
+	int f_count2_index;
 	/*
 	 * This function reads the relevant file and fill the buffer
 	 * with statistics corresponding to given activity.
@@ -979,8 +975,7 @@ struct activity {
 	/*
 	 * This function is used by sadf to display activity statistics in PCP format.
 	 */
-	__print_funct_t (*f_pcp_print) (struct activity *, int, unsigned long long,
-					struct record_header *);
+	__print_funct_t (*f_pcp_print) (struct activity *, int);
 	/*
 	 * This function is used by sadf to count the number of new items in current
 	 * sample and add them to the linked list @item_list.
@@ -1003,6 +998,9 @@ struct activity {
 	 * separated with a '|' character.
 	 * For a given output, the first field corresponding to extended statistics
 	 * (eg. -r ALL) begins with a '&' character.
+	 * A field having '*' in its name means that all items will have its own column
+	 * (the first one will be displayed as "all", the following ones will get the '*'
+	 * character replaced with their item number (0, 1, etc.)
 	 */
 	char *hdr_line;
 	/*
@@ -1348,6 +1346,8 @@ struct sa_item {
 /*
  * Prototypes used to count new items
  */
+__nr_t count_new_int
+	(struct activity *, int);
 __nr_t count_new_net_dev
 	(struct activity *, int);
 __nr_t count_new_net_edev
@@ -1531,6 +1531,8 @@ char *get_fs_name_to_display
 	(struct activity *, uint64_t, struct stats_filesystem *);
 unsigned long long get_global_cpu_statistics
 	(struct activity *, int, int, uint64_t, unsigned char []);
+void get_global_int_statistics
+	(struct activity *, int, int, uint64_t, unsigned char []);
 void get_global_soft_statistics
 	(struct activity *, int, int, uint64_t, unsigned char []);
 void get_itv_value
@@ -1540,11 +1542,9 @@ void init_custom_color_palette
 int next_slice
 	(unsigned long long, unsigned long long, int, long);
 void parse_sa_devices
-	(char *, struct activity *, int, int *, int);
+	(char *, struct activity *, int, int *, int, int);
 int parse_sar_opt
 	(char * [], int *, struct activity * [], uint64_t *, int);
-int parse_sar_I_opt
-	(char * [], int *, uint64_t *, struct activity * []);
 int parse_sa_P_opt
 	(char * [], int *, uint64_t *, struct activity * []);
 int parse_sar_m_opt
