@@ -1,6 +1,6 @@
 /*
  * sadf: system activity data formatter
- * (C) 1999-2021 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2023 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -108,7 +108,7 @@ void usage(char *progname)
 	fprintf(stderr, _("Options are:\n"
 			  "[ -C ] [ -c | -d | -g | -j | -l | -p | -r | -x ] [ -H ] [ -h ] [ -T | -t | -U ] [ -V ]\n"
 			  "[ -O <opts> [,...] ] [ -P { <cpu> [,...] | ALL } ]\n"
-			  "[ --dev=<dev_list> ] [ --fs=<fs_list> ] [ --iface=<iface_list> ]\n"
+			  "[ --dev=<dev_list> ] [ --fs=<fs_list> ] [ --iface=<iface_list> ] [ --int=<int_list> ]\n"
 			  "[ -s [ <hh:mm[:ss]> ] ] [ -e [ <hh:mm[:ss]> ] ]\n"
 			  "[ -- <sar_options> ]\n"));
 	exit(1);
@@ -335,8 +335,9 @@ void list_fields(unsigned int act_id)
 	unsigned int msk;
 	char *hl;
 	char hline[HEADER_LINE_LEN] = "";
+	char out[256];
 
-	printf("# hostname;interval;timestamp");
+	cprintf_s(IS_COMMENT, "%s", "# hostname;interval;timestamp");
 
 	for (i = 0; i < NR_ACT; i++) {
 
@@ -345,9 +346,10 @@ void list_fields(unsigned int act_id)
 
 		if (IS_SELECTED(act[i]->options) && (act[i]->nr_ini > 0)) {
 			if (!HAS_MULTIPLE_OUTPUTS(act[i]->options)) {
-				printf(";%s", act[i]->hdr_line);
+				sprintf(out, ";%s", act[i]->hdr_line);
+				cprintf_s(IS_COMMENT, "%s", out);
 				if ((act[i]->nr_ini > 1) && DISPLAY_HORIZONTALLY(flags)) {
-					printf("[...]");
+					cprintf_s(IS_COMMENT, "%s", "[...]");
 				}
 			}
 			else {
@@ -361,20 +363,23 @@ void list_fields(unsigned int act_id)
 							if ((act[i]->opt_flags & 0xff00) & (msk << 8)) {
 								/* Display whole header line */
 								*(hl + j) = ';';
-								printf(";%s", hl);
+								sprintf(out, ";%s", hl);
+								cprintf_s(IS_COMMENT, "%s", out);
 							}
 							else {
 								/* Display only the first part of the header line */
 								*(hl + j) = '\0';
-								printf(";%s", hl);
+								sprintf(out, ";%s", hl);
+								cprintf_s(IS_COMMENT, "%s", out);
 							}
 							*(hl + j) = '&';
 						}
 						else {
-							printf(";%s", hl);
+							sprintf(out, ";%s", hl);
+							cprintf_s(IS_COMMENT, "%s", out);
 						}
 						if ((act[i]->nr_ini > 1) && DISPLAY_HORIZONTALLY(flags)) {
-							printf("[...]");
+							cprintf_s(IS_COMMENT, "%s", "[...]");
 						}
 					}
 				}
@@ -398,10 +403,10 @@ void list_fields(unsigned int act_id)
 time_t get_time_ref(void)
 {
 	struct tm ltm;
-	time_t t;
+	time_t t = record_hdr[2].ust_time;
 
 	if (DISPLAY_ONE_DAY(flags)) {
-		localtime_r((time_t *) &(record_hdr[2].ust_time), &ltm);
+		localtime_r(&t, &ltm);
 
 		/* Move back to midnight */
 		ltm.tm_sec = ltm.tm_min = ltm.tm_hour = 0;
@@ -659,7 +664,7 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 	if (reset_cd) {
 		/*
 		 * See note in sar.c.
-		 * NB: Reseting cross_day is needed only if datafile
+		 * NB: Resetting cross_day is needed only if datafile
 		 * may be rewinded (eg. in db or ppc output formats).
 		 */
 		cross_day = 0;
@@ -752,8 +757,12 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 			else if (format == F_RAW_OUTPUT) {
 				/* Raw output */
 				if (DISPLAY_DEBUG_MODE(flags)) {
-					printf("# name; %s; nr_curr; %d; nr_alloc; %d; nr_ini; %d\n", act[i]->name,
-					       act[i]->nr[curr], act[i]->nr_allocated, act[i]->nr_ini);
+					char out[128];
+
+					sprintf(out, "# name; %s; nr_curr; %d; nr_alloc; %d; nr_ini; %d\n",
+						act[i]->name, act[i]->nr[curr], act[i]->nr_allocated,
+						act[i]->nr_ini);
+					cprintf_s(IS_COMMENT, "%s", out);
 				}
 
 				if (IS_SELECTED(act[i]->options) && (act[i]->nr[curr] > 0)) {
@@ -764,7 +773,7 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 			else if (format == F_PCP_OUTPUT) {
 				/* PCP archive */
 				if (*act[i]->f_pcp_print) {
-					(*act[i]->f_pcp_print)(act[i], curr, itv, &record_hdr[curr]);
+					(*act[i]->f_pcp_print)(act[i], curr);
 				}
 			}
 
@@ -1527,16 +1536,7 @@ int main(int argc, char **argv)
 	/* Process options */
 	while (opt < argc) {
 
-		if (!strcmp(argv[opt], "-I")) {
-			if (!sar_options) {
-				usage(argv[0]);
-			}
-			if (parse_sar_I_opt(argv, &opt, &flags, act)) {
-				usage(argv[0]);
-			}
-		}
-
-		else if (!strcmp(argv[opt], "-P")) {
+		if (!strcmp(argv[opt], "-P")) {
 			if (parse_sa_P_opt(argv, &opt, &flags, act)) {
 				usage(argv[0]);
 			}
@@ -1545,25 +1545,31 @@ int main(int argc, char **argv)
 		else if (!strncmp(argv[opt], "--dev=", 6)) {
 			/* Parse devices entered on the command line */
 			p = get_activity_position(act, A_DISK, EXIT_IF_NOT_FOUND);
-			parse_sa_devices(argv[opt], act[p], MAX_DEV_LEN, &opt, 6);
+			parse_sa_devices(argv[opt], act[p], MAX_DEV_LEN, &opt, 6, NO_RANGE);
 		}
 
 		else if (!strncmp(argv[opt], "--fs=", 5)) {
 			/* Parse devices entered on the command line */
 			p = get_activity_position(act, A_FS, EXIT_IF_NOT_FOUND);
-			parse_sa_devices(argv[opt], act[p], MAX_FS_LEN, &opt, 5);
+			parse_sa_devices(argv[opt], act[p], MAX_FS_LEN, &opt, 5, NO_RANGE);
 		}
 
 		else if (!strncmp(argv[opt], "--iface=", 8)) {
 			/* Parse devices entered on the command line */
 			p = get_activity_position(act, A_NET_DEV, EXIT_IF_NOT_FOUND);
-			parse_sa_devices(argv[opt], act[p], MAX_IFACE_LEN, &opt, 8);
+			parse_sa_devices(argv[opt], act[p], MAX_IFACE_LEN, &opt, 8, NO_RANGE);
 			q = get_activity_position(act, A_NET_EDEV, EXIT_IF_NOT_FOUND);
 			act[q]->item_list = act[p]->item_list;
 			act[q]->item_list_sz = act[p]->item_list_sz;
 			act[q]->options |= AO_LIST_ON_CMDLINE;
 		}
 
+		else if (!strncmp(argv[opt], "--int=", 6)) {
+			/* Parse interrupts names entered on the command line */
+			p = get_activity_position(act, A_IRQ, EXIT_IF_NOT_FOUND);
+			parse_sa_devices(argv[opt], act[p], MAX_SA_IRQ_LEN, &opt, 6, NR_IRQS);
+		}
+
 		else if (!strcmp(argv[opt], "-s")) {
 			/* Get time start */
 			if (parse_timestamp(argv, &opt, &tm_start, DEF_TMSTART)) {
@@ -1848,6 +1854,9 @@ int main(int argc, char **argv)
 		}
 	}
 
+	/* Init color strings */
+	init_colors();
+
 	if (USE_OPTION_A(flags)) {
 		/* Set -P ALL -I ALL if needed */
 		set_bitmaps(act, &flags);
